
//--------------------------------------------------------------------------------------
// File: shader02.fx
// position, normal, WVPBuffer
//--------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------
// Constant Buffer Variables
//--------------------------------------------------------------------------------------
cbuffer WVPBuffer : register( b0 )
{
	matrix world;
	matrix view;
	matrix projection;
	float4 m_ambient_color;
}
//--------------------------------------------------------------------------------------
struct VS_INPUT
{
    float3 pos : POSITION;
    float3 normal : NORMAL;
//    float2 tex : TEXCOORD0;
};

struct VS_INSTANCE_INPUT
{
    float3 pos : POSITION;
    float3 normal : NORMAL;
    float3 pos_ins : INS_POS;
};

struct PS_INPUT
{
    float4 pos : SV_POSITION;
    float3 normal : NORMAL;
//    float3 color : COLOR;
};

//--------------------------------------------------------------------------------------
// Vertex Shader
//--------------------------------------------------------------------------------------
PS_INPUT VS( VS_INPUT input )
{
	PS_INPUT output;
	output.pos = float4(input.pos, 1.0f);
    output.pos = mul( output.pos, world );
    output.pos = mul( output.pos, view );
    output.pos = mul( output.pos, projection );
    output.normal = mul( input.normal, world );
//	output.color = float3(1.0f, 0.0f, 1.0f);    
    return output;
}

PS_INPUT VS_INSTANCE( VS_INSTANCE_INPUT input )
{
	PS_INPUT output;
	output.pos = float4(input.pos + input.pos_ins, 1.0f);
    output.pos = mul( output.pos, world );
    output.pos = mul( output.pos, view );
    output.pos = mul( output.pos, projection );
    output.normal = mul( input.normal, world );
//	output.color = float3(1.0f, 1.0f, 1.0f);
    return output;
}

//--------------------------------------------------------------------------------------
// Pixel Shader
//--------------------------------------------------------------------------------------
float4 PS( PS_INPUT input ) : SV_Target
{
	static const float3 lightDirection = {0.288675f, -0.288675f, 0.288675f};	//(0.5 * 3^{-0.5})
	static const float3 materialColor = {1.0f, 0.0f, 1.0f};
	
	float scale = 0.5f - dot(input.normal, lightDirection);
	scale = max(scale, 0.0f);
	float4 color = float4(m_ambient_color.xyz * scale, m_ambient_color.w);
	
    return color;
}

float4 PS_INSTANCE( PS_INPUT input ) : SV_Target
{
	static const float3 lightDirection = {0.288675f, -0.288675f, 0.288675f};	//(0.5 * 3^{-0.5})
	static const float3 materialColor = {1.0f, 1.0f, 1.0f};
	
	float scale = 0.5f - dot(input.normal, lightDirection);
	scale = max(scale, 0.0f);
	float4 color = float4(m_ambient_color.xyz * scale, m_ambient_color.w);
	
    return color;
}
